#include <avr/io.h>
#include "archdep.h"
#include "uart.h"

void uart_init(void)
{
    uint8_t sreg = SREG;
    uint16_t ubrr =  ((F_CPU+(USART_BAUD*8L))/(USART_BAUD*16L)-1); 
    UBRRH = ubrr >> 8;
    UBRRL = ubrr;
    UCSRB = (1 << TXEN);
    SREG = sreg;
}




inline int uart_putc(const uint8_t c)
{
    loop_until_bit_is_set(UCSRA, UDRE);
    UDR = c;
    return 0;
}

inline void uart_puts(const char *s)
{
    do {
        uart_putc(*s);
    }
    while (*s++);
}

inline void uart_puts_P(PGM_P s)
{
    while (1) {
        unsigned char c = pgm_read_byte(s);
        s++;
        if ('\0' == c)
            break;
        uart_putc(c);
    }
}
const char *itoh(uint32_t x)
{
    /* Nasty hack to convert 32 bit unsigned value
       to a string of 8 hex digits.
       Static allocation, so it's not re-entrant
     */
    uint8_t d;
    uint32_t i;
    static char s[10];
    s[8] = 0;

    for (i = 0; i < 8; i++) {
        d = ((x >> ((7 - i) * 4)) & 0x0f);

        if (d <= 9)
            s[i] = d + '0';
        else
            s[i] = d - 0x0a + 'A';
    }
    s[9] = 0;
    return s;
}

