/*************************************************************
* main.c - openwii main.c
* Copyright YOLD 3173 OpenWii all rights reversed.
**************************************************************/

#include "common.h"
#include <stdlib.h>

#ifdef CHIIP
#include "../chiip/uart.h"
#endif

/*overcomplicated led related stuff*/
uint16_t ledcnt0; /*divisor for led*/
uint16_t ledspeed0; /*Current speed in ms*/

uint16_t ledcnt1; /*divisor for led*/
uint16_t ledspeed1; /*Current speed in ms*/


/*inline used by the timer interrupt for flashing the led.
* This function should be called every ms
* The ledspeed0 global is then used to set the flash rate 
* 0=off. heh*/
inline void msTimerInt() {
	if((ledspeed0!=0) && (++ledcnt0>=ledspeed0)) { //if == 0xff then noflash
	 	LEDTOG0;
		ledcnt0=0;	
	}
	if((ledspeed1!=0) && (++ledcnt1>=ledspeed1)) { //if == 0xff then noflash
	 	LEDTOG1;
		ledcnt1=0;	
	}

}


/*teh main!*/
int main()
{
	int result;
	uint16_t login_state;	
	uint16_t media_flag;	
	uint16_t d2a_check;
	uint32_t p_loginState;	/*These are used to change stuff between d2a and dms*/
	uint32_t p_mediaFlag;	/*thought of using a table but unless more firmware */
	uint32_t p_unlockAddr;	/*versions or addresses are used this is prolly most*/
	uint32_t p_emuFlag;	/*efficient way*/
	uint32_t p_otherFlag;
	uint32_t p_stackRet;
	uint32_t p_discsize1;
	uint32_t p_discsize2;
	uint32_t p_discsize3;
	uint32_t p_discsize4;
	uint16_t org_ret;
	uint16_t new_ret;
	uint8_t backup;
	uint16_t maxsectors;
	uint16_t ret_word;



	machInit();
	portInit();
	ledspeed0=20;	//start flashing slowly to indicate trying to sync
	ledcnt0=0;
	LEDON0;		    //turn led on
	ledspeed1=50;	//start flashing slowly to indicate trying to sync
	ledcnt1=0;
	LEDON1;		    //turn led on
    SetupTimer();
	SetupSerial();
	intInit();
    #ifdef CHIIP
    uart_init();
    uart_puts_P(PSTR("Init done " CR));
    #endif

	while(1)
	{
	    uart_puts_P(PSTR("Wait for Sync" CR));
  	    uart_puts_P(PSTR("login_state=0x"));
        uart_puts(itoh(99));
        uart_puts_P(CR);
 
        if( Sync() == 0)
		{
			uart_puts_P(PSTR("Got Sync" CR));
            ledspeed0=500;	//flash around 1hz (500*2)=1hz), sync acquired
			break;
		}
        ledspeed0=50;
		waitus(3000);
		//waitus(30000);
    }


	/*Check for drive type*/
	uart_puts_P(PSTR("Get d2a_check" CR));
	SerialRequest(CMD_READ, 0x40BCA8, 2, 0x00, &d2a_check);
	if(d2a_check==D2ACHK) {	/*set pointers for d2a/dms*/
		uart_puts_P(PSTR("Setup d2a" CR));
        p_loginState=0x40ba06;
		p_mediaFlag=0x8576;
		p_unlockAddr=0x82b6;
		p_emuFlag=0x8598;
		p_otherFlag = 0x40bd7a;
		p_stackRet = 0x8f84;
		p_discsize1 = 0x8582;
		p_discsize2 = 0x8578;
		p_discsize3 = 0x8586;
		p_discsize4 = 0x857c;
		org_ret=0xd4c7;
		new_ret=0xd4db;				
	} else {	/*set pointers for d2b*/
		uart_puts_P(PSTR("Setup d2b" CR));
        p_loginState=0x40ba06;
		p_mediaFlag=0x8580;
		p_unlockAddr=0x82be;
		p_emuFlag=0x85A2;
		p_otherFlag=0x40BD86;
		p_stackRet = 0x8f84;
		p_discsize1 = 0x8582;
		p_discsize2 = 0x8586;
		p_discsize3 = 0x858C;
		p_discsize4 = 0x8590;
		org_ret=0xd54a;
		new_ret=0xd55e;
	}

	while(1) {
		uart_puts_P(PSTR("WRITE unlockAddr 0x4100" CR));
		result = SerialRequest(CMD_WRITE, p_unlockAddr, 2, 0x4100, NULL);
		if(result != 0){
		    uart_puts_P(PSTR("Resync" CR));
			ledspeed0 = 50;
            Resync();
    		//waitus(30000);
    
        } else
			break;
	}

	backup = 0;
    ledspeed1 = 50;
    	
	while(1) {
 	    ledspeed0 = 500;
        result = SerialRequest(CMD_READ, p_loginState, 1, 0x0, &login_state);
		if(result != 0) {
            uart_puts_P(PSTR("Resync" CR));
            ledspeed0 = 50;
			Resync();
			//waitus(30000);
    	}else{
        
            uart_puts_P(PSTR("login_state=0x"));
            uart_puts(itoh(login_state));
            uart_puts_P(CR);
            
			if(login_state == 0x7F)
			{
                uart_puts_P(PSTR("WRITE unlockAddr 0x41" CR));
                SerialRequest(CMD_WRITE, p_unlockAddr, 1, 0x41, NULL);
	            uart_puts_P(PSTR("READ stackRet 0x00" CR));
 			    SerialRequest(CMD_READ, p_stackRet, 2, 0x0, &ret_word);
				if(ret_word == org_ret) {
					uart_puts_P(PSTR("WRITE stackRet" CR));
                    SerialRequest(CMD_WRITE, p_stackRet, 2, new_ret, NULL);
				}
			}

			if( (login_state >= 0x7F) && (login_state < 0xe0)) // >=0x7F ?
			{
                uart_puts_P(PSTR("WRITE unlockAddr 0x41" CR));
                SerialRequest(CMD_WRITE, p_unlockAddr, 1, 0x41, NULL);
		        uart_puts_P(PSTR("READ media_flag 0x0000" CR));
 				SerialRequest(CMD_READ, p_mediaFlag, 2, 0x0, &media_flag);
				uart_puts_P(PSTR("media_flag=0x"));
                uart_puts(itoh(media_flag));
                uart_puts_P(CR);
                if( (media_flag == 0x1018) || (media_flag == 0x0818) || (media_flag == 0x200c ))
				{
				    uart_puts_P(PSTR("WRITE media_flag 0x8828" CR));
                    SerialRequest(CMD_WRITE, p_mediaFlag, 2, 0x8828, NULL);
					uart_puts_P(PSTR("WRITE emuFlag 0x04" CR));
    				SerialRequest(CMD_WRITE, p_emuFlag, 2, 0x4, NULL);
					backup = 1;
                    ledspeed1 = 500;
				    uart_puts_P(PSTR("backup = 1" CR));
    
					SerialRequest(CMD_READ, 0x408817, 1, 0x0, &d2a_check);
					maxsectors=d2a_check;
					SerialRequest(CMD_READ, 0x408816, 1, 0x0, &d2a_check);
					maxsectors+=d2a_check<<8;
					SerialRequest(CMD_READ, 0x408815, 1, 0x0, &d2a_check);
					SerialRequest(CMD_WRITE, p_discsize1, 2, maxsectors, NULL);
					SerialRequest(CMD_WRITE, p_discsize2, 2, maxsectors, NULL);
					SerialRequest(CMD_WRITE, p_discsize3, 2, maxsectors, NULL);
					SerialRequest(CMD_WRITE, p_discsize4, 2, maxsectors, NULL);

					SerialRequest(CMD_WRITE, p_discsize1+2, 1, d2a_check, NULL);
					SerialRequest(CMD_WRITE, p_discsize2+2, 1, d2a_check, NULL);
					SerialRequest(CMD_WRITE, p_discsize3+2, 1, d2a_check, NULL);
					SerialRequest(CMD_WRITE, p_discsize4+2, 1, d2a_check, NULL);
				}
			}
			if((login_state >= 0xe0) && (backup == 1)) {
				uart_puts_P(PSTR("WRITE media_flag 0x8838" CR));
                SerialRequest(CMD_WRITE, p_mediaFlag, 2, 0x8838, NULL);
				ledspeed0=200;
			}

			// Reset our backup flag after F0
			if((login_state >= 0xF4)) {
                if(backup == 1){
					uart_puts_P(PSTR("WRITE otherFlag 0xBD01" CR));
    				SerialRequest(CMD_WRITE, p_otherFlag, 2, 0xBD01, NULL);
                }
                backup=0;
                ledspeed1 = 50;
			}
		}
    	//waitus(30000);
    }	
}
